/************************************************************
		azimuth_chirp.c
	   	
  Prepare the conjugate of the synthetic azimuth chirp signal. 

	1. FFT compatible format (periodic)
	2. Middle of the chirp centred at origin, s=0)


	<< Conjugate of the azimuth Chirp >>
	   
	   h(t)=exp[-2*pi(f_Dc*s+f_r*s*s/2)], |s|<S/2
	  
	   where    
		   f_Dc: Doppler_centroid 
		    f_r: Doppler_rate 
	 	      S: Coherent integration time	


		<< Data Structure>>

	    *****...****0   ... 0 ...     0*****...****
	    -------------------------------------------
slow	    |          |		   |          |  
 time(s):   0         S/2  		 -S/2     -delta_s		

azimuth_chirp          |       		   |	      |               
_data[i]:   0      chirp_end	    chirp_start  nazimuth_fft-1 


by Dr. Hoonyol Lee
Department of Geophysics
Kangwon National University

f_Dc deleted (will be compensated during migration): 1 Dec 2004
Initiated on 7 May 2004
*************************************************************/

void azimuth_chirp()
{
	int i, chirp_end, chirp_start; 
	double s, phase;

	printf(" azimuth_chirp:\n");

	azimuth_chirp_data = dvector(nazimuth_fft*2);

	chirp_end = (int)(coherent_integration_time/2./delta_s + 0.5);
	chirp_start = nazimuth_fft + 
	 	(int)(coherent_integration_time/(-2.)/delta_s + 0.5);

	/* first half of nazimuth_fft */ 
	for(i=0; i<chirp_end; i++) 
	{
		s= i*delta_s; 	
		/*phase= 2*pi*(Doppler_centroid*s + Doppler_rate*s*s/2.);
		*/
		phase= pi*Doppler_rate*s*s;
		azimuth_chirp_data[2*i]=cos(phase);
		azimuth_chirp_data[2*i+1]= (-1.)*sin(phase); /* conjugate */
	}

	/* next half of nazimuth_fft */
	for(i=chirp_start; i<nazimuth_fft; i++) 
	{
		s = (i-nazimuth_fft)*delta_s; 	
		/*phase = 2*pi*(Doppler_centroid*s + Doppler_rate*s*s/2.);
		*/
		phase = pi*Doppler_rate*s*s;
		azimuth_chirp_data[2*i]=cos(phase);
		azimuth_chirp_data[2*i+1]= (-1.)*sin(phase); /* conjugate */
	}

}

 	
