/********************************************************
		cfout.c

	Complex File OUTput program 

		* * *

  cfout(data, nrow, ncol, istyle, iband, ireal, ftail)

	1. data (input data): double, IQ, BIP 
	2. nrow: number of row
	3. ncol: number of col
	4. istyle (output style): "IQ"(0)  or "amp/phase"(1)
  	5. iband (output band format): BIP(0), BIL(1), BSQ(2)	
	6. ireal (output data format): float(0) or double(1)
 	7. ftail: filename tail (fname=fhead+ftail)			 

Currently istyle=(0,1), iband=0, ireal=0 are available.

by Dr. Hoonyol Lee
Department of Geophysics
Kangwon National University

* BIL enabled on 28 Nov 2004
* coding initiated on 7 May 2004
*********************************************************/	

void cfout(cdata, nrow, ncol, istyle, iband, 
ireal, ftail)
double *cdata;
int nrow, ncol, istyle, iband, ireal;
char *ftail;
{

	FILE *fp;
	char fname[80];
	int i, j, ix;
	double *dbuf, x1, y1, amplitude, phase; 
    	float *fbuf;
 
	strcpy(fname, fhead);
	strcat(fname, "_");
	strcat(fname, ftail);

	printf("   cfout: %s", fname);

	if((fp=fopen(fname, "w"))==NULL)
	{
		printf(" (Error) Can not open %s.\n", fname);
		exit(0);	
	}

	if(istyle==0) /* "IQ" */
	{
	   if(iband==0) /* BIP */
	   {
		if(ireal==0)
		{
		   printf(" (IQ, BIP, FLOAT)\n");
		   fbuf=fvector(ncol*2);
		   for(i=0; i<nrow; i++)
		   {
		      	ix=i*ncol*2;
		 	for(j=0; j<ncol*2; j++) fbuf[j]=(float)cdata[ix+j];
		 	fwrite(fbuf, 1, ncol*sizeof(float)*2, fp);
		   }
		   free(fbuf);
		}
		else
		{
		   printf(" (IQ, BIP, DOUBLE)\n");
		   fwrite(cdata, 1, nrow*ncol*2*sizeof(double), fp);
		}
	   }
	   else if(iband==1) /* BIL */
	   {
		if(ireal==0)
		{
		   printf(" (IQ, BIL, FLOAT)\n");
		   fbuf=fvector(ncol*2);
		   for(i=0; i<nrow; i++)
		   {
		      	ix=i*ncol*2;
		 	for(j=0; j<ncol; j++)
			{
				fbuf[j]=(float)cdata[ix+2*j];
			 	fbuf[ncol+j]=(float)cdata[ix+2*j+1];	
			}
		 	fwrite(fbuf, 1, ncol*sizeof(float)*2, fp);
		   }
		   free(fbuf);
		   if(oerm==1) gen_ers(fname, nrow, ncol, azimuth_pixel_size,
			ground_range_pixel_size, 2, "141"); 
		}
		else
		{
		   printf(" (IQ, BIL, DOUBLE)\n");
		   dbuf=dvector(ncol*2);
		   for(i=0; i<nrow; i++)
		   {
		      	ix=i*ncol*2;
		 	for(j=0; j<ncol; j++)
			{
				dbuf[j]=cdata[ix+2*j];
			 	dbuf[ncol+j]=cdata[ix+2*j+1];	
			}
		 	fwrite(dbuf, 1, ncol*sizeof(double)*2, fp);
		   }
		   free(dbuf);
		   if(oerm==1) gen_ers(fname, nrow, ncol, azimuth_pixel_size,
			ground_range_pixel_size, 2, "181"); 
		}
	   }
	   else /* BSQ */
	   {
		printf(" (Error) Sorry this format is not ready yet.\n");
		exit(0);
	   }
	}
	else /* "amplitude, phase" */
	{
	   if(iband==0) /* BIP */
	   {
	     if(ireal==0) /* FLOAT */
	     {
		printf(" (AP, BIP, FLOAT)\n");
		fbuf = fvector(ncol*2);
		for(i=0; i<nrow; i++)
		{
		   ix=i*ncol*2;
		   for(j=0; j<ncol; j++)
		   {
		   	x1=cdata[ix+2*j];
			y1=cdata[ix+2*j+1];

		   	amplitude=sqrt(x1*x1+y1*y1);
			phase=atan2(y1, x1);

			fbuf[2*j]=(float)amplitude;
			fbuf[2*j+1]=(float)phase;
		   }
		   fwrite(fbuf, 1, ncol*sizeof(float)*2, fp);
		}
		free(fbuf);
	     }
	     else /* DOUBLE */
	     {
		printf(" (AP, BIP, DOUBLE)\n");
		dbuf = dvector(ncol*2);
		for(i=0; i<nrow; i++)
		{
		   ix=i*ncol*2;
		   for(j=0; j<ncol; j++)
		   {
		   	x1=cdata[ix+2*j];
			y1=cdata[ix+2*j+1];

		   	amplitude=sqrt(x1*x1+y1*y1);
			phase=atan2(y1, x1);

			dbuf[2*j]=amplitude;
			dbuf[2*j+1]=phase;
		   }
		   fwrite(dbuf, 1, ncol*sizeof(double)*2, fp);
		}
		free(dbuf);
	     }
	   }
	   else if(iband==1) /* BIL */ 
	   {
	     if(ireal==0) /* FLOAT */
	     {
		printf(" (AP, BIL, FLOAT)\n");
		fbuf = fvector(ncol*2);
		for(i=0; i<nrow; i++)
		{
		   ix=i*ncol*2;
		   for(j=0; j<ncol; j++)
		   {
		   	x1=cdata[ix+2*j];
			y1=cdata[ix+2*j+1];

		   	amplitude=sqrt(x1*x1+y1*y1);
			phase=atan2(y1, x1);

			fbuf[j]=(float)amplitude;
			fbuf[ncol+j]=(float)phase;
		   }
		   fwrite(fbuf, 1, ncol*sizeof(float)*2, fp);
		}
		free(fbuf);
	 	if(oerm==1) gen_ers(fname, nrow, ncol, azimuth_pixel_size,
			ground_range_pixel_size, 2, "141"); 
	     }
	     else
	     {
		printf(" (AP, BIL, DOUBLE)\n");
		dbuf = dvector(ncol*2);
		for(i=0; i<nrow; i++)
		{
		   ix=i*ncol*2;
		   for(j=0; j<ncol; j++)
		   {
		   	x1=cdata[ix+2*j];
			y1=cdata[ix+2*j+1];

		   	amplitude=sqrt(x1*x1+y1*y1);
			phase=atan2(y1, x1);

			dbuf[j]=amplitude;
			dbuf[j+ncol]=phase;
		   }
		   fwrite(dbuf, 1, ncol*sizeof(double)*2, fp);
		}
		free(dbuf);
	 	if(oerm==1) gen_ers(fname, nrow, ncol, azimuth_pixel_size,
			ground_range_pixel_size, 2, "181"); 
	     }
	   }
	   else /* BSQ */
	   {
		printf(" (Error) Sorry BSQ format is not supported yet.\n");
		exit(0);
	   }
	}

	fclose(fp);

}

