/****************************************************************** 
	      	exraw.c  

	    Extract SAR RAW Data	  

1. This program extracts a subscene from SAR raw data.  
2. Make a parameter file (eg. exraw.pam) before you run this program.
3. The input file format should be unsigned character with IQ in BIP,
  which is the case for most commercial satellite SAR raw data.
4. The output file format is also unsigned char and IQ in BIP. 

Written by Dr. Hoonyol Lee, May 03 2004. 
******************************************************************/ 

#include <stdio.h>
#include "ma.c"
 
void usage();

main(int argc, char *argv[])
{
	FILE *fp, *fp_in, *fp_out;
	int nrange_whole, nazimuth_whole;
	int nbyte_per_record, nskip_record, nskip_byte_per_record;
	int range_start, range_end, nrange;
	int azimuth_start, azimuth_end, nazimuth;

	int i, j;	
	char *buf, vbuf[80], fname_in[80], fname_out[80];
	int nramnant_byte_per_record;
	int total_byte_read, total_byte_written, nread, nwritten;
	int np; 

	printf(" Program exraw: extract subscene from SAR RAW data\n\n");
	if(argc<2){usage(); exit(0);}

	/* Open Parameter File */
	if((fp=fopen(argv[1], "r"))==NULL)
	{	printf(" (Error) Cannot open %s.\n\a", argv[1]);
		exit(0);
	}

	/* Read Parameter File */
	fscanf(fp, "%s %s", vbuf, fname_in);
	printf("  %s= %s\n", vbuf, fname_in);

	fscanf(fp, "%s %s", vbuf, fname_out);
	printf("  %s= %s\n", vbuf, fname_out);
	
	fscanf(fp, "%s %d", vbuf, &nrange_whole);
	printf("  %s= %d\n", vbuf, nrange_whole);
	
	fscanf(fp, "%s %d", vbuf, &nazimuth_whole);
	printf("  %s= %d\n", vbuf, nazimuth_whole);

	fscanf(fp, "%s %d", vbuf, &nbyte_per_record);
	printf("  %s= %d\n", vbuf, nbyte_per_record);

	fscanf(fp, "%s %d", vbuf, &nskip_record);
	printf("  %s= %d\n", vbuf, nskip_record);

	fscanf(fp, "%s %d", vbuf, &nskip_byte_per_record);
	printf("  %s= %d\n", vbuf, nskip_byte_per_record);

	fscanf(fp, "%s %d", vbuf, &range_start);
	printf("  %s= %d\n", vbuf, range_start);

	fscanf(fp, "%s %d", vbuf, &range_end);
	printf("  %s= %d\n", vbuf, range_end);
	
	fscanf(fp, "%s %d", vbuf, &nrange);
	printf("  %s= %d\n", vbuf, nrange);
	
	fscanf(fp, "%s %d", vbuf, &azimuth_start);
	printf("  %s= %d\n", vbuf, azimuth_start);

	fscanf(fp, "%s %d", vbuf, &azimuth_end);
	printf("  %s= %d\n", vbuf, azimuth_end);

	fscanf(fp, "%s %d", vbuf, &nazimuth);
	printf("  %s= %d\n", vbuf, nazimuth);


	/* Memoy Allocation */
	buf = ucvector(nrange*2);

	/* Open Input and Output file */
	if((fp_in=fopen(fname_in, "r"))==NULL)
	{ 	printf(" (Error) Cannot open %s.\n\a", fname_in);
		exit(0);
	}

	if((fp_out=fopen(fname_out, "w"))==NULL)
	{ 	printf(" (Error) Cannot open %s.\n\a", fname_out);
		exit(0);
	}


	/* Read and Write */
	
	/*initial+ azimuth record skip*/
	fseek(fp_in, nbyte_per_record*(nskip_record+azimuth_start),
	 	SEEK_SET); 

	nramnant_byte_per_record = nbyte_per_record
		- nskip_byte_per_record - nrange*2;
	total_byte_read=0;
	total_byte_written=0;
	for(i=0; i<nazimuth; i++)                 
	{
		/* skip first byte per record*/
		fseek(fp_in, nskip_byte_per_record, SEEK_CUR); 

		/* read range data */
		nread=fread(buf, 1, nrange*2, fp_in);
		total_byte_read += nread;

		/* write range data */	
		nwritten=fwrite(buf, 1, nrange*2, fp_out);
		total_byte_written += nwritten;

		/* skip rest of the range per record */ 
		fseek(fp_in, nramnant_byte_per_record, SEEK_CUR); 
	}

	fclose(fp_in);
	fclose(fp_out);

	/* Check Bytes */
	if(total_byte_written != nrange*nazimuth*2)
	{	
		printf(" (Warning) %d bytes written out of %d.\n",
			total_byte_written, nrange*nazimuth*2);
	} else
	{ 
	        printf(" Total %d bytes were extracted.\n", total_byte_written);
		printf(" The output file %s is created.\n", fname_out);
		printf(" The program has finished successfully.\n");
	}
}

void usage()
{
	printf(" Usage: exraw parameter_file \n\a");
}




