/***************************************************************** 
   load_data

	1. load data  from unsigned char, IQ in BIP RAW (FFT ready). 
	2. Nominal_mean and DC_bias correction.

by Dr. Hoonyol Lee 
Dept. of Geophysics
Kangwon National Univeristy
7 May 2004

22 May 2004: 
	1. exraw.c merged.
	2. critical debug - range_start.
*****************************************************************/ 

void load_data()
{
  	FILE *fp_in, *fp_out;
	char fname_out[80];	
	int i, j, data_pointer, ix;
	unsigned char *buf;
	int total_byte_read, total_byte_written, nread, nwritten;
	int nremnant_byte_per_record;

	printf("\n load_data:\n");
	printf("  Extract SAR RAW data. \n");

	buf=ucvector(nrange*2);

	if((fp_in=fopen(fname_in, "r"))==NULL)
	{
		printf(" (Error) Cannot open %s.\n\a", fname_in);
		exit(0);
	}

	if(oflag[0]>1)
	{ 
	   strcpy(fname_out, fhead);
	   strcat(fname_out, "_00_RAW");
	   if((fp_out=fopen(fname_out, "w"))==NULL)
	   { 	printf(" (Error) Cannot open %s.\n\a", fname_out);
		exit(0);
	   } else
	   {
		printf("  SAR RAW data output %s.\n", fname_out);	
	   }
	}

	/*initial + azimuth record skip*/
	fseek(fp_in, nbyte_per_record*(nskip_record+azimuth_start),
	 	SEEK_SET); 
	
	nremnant_byte_per_record = nbyte_per_record
		- nskip_byte_per_record - range_start*2 - nrange*2;

	total_byte_read=0;
	total_byte_written=0;
	data_pointer=0;
	for(i=0; i<nazimuth; i++)                 
	{
		/* skip first byte per record*/
		fseek(fp_in, nskip_byte_per_record + range_start*2, SEEK_CUR); 

		/* read range data */
		nread=fread(buf, 1, nrange*2, fp_in);
		total_byte_read += nread;

		/* prepare double data in FFT-ready format */
		for(j=0; j<nrange*2; j++)
		   	data[data_pointer+j] = (double)buf[j];
		data_pointer += nrange_fft*2; 

		/* write range data */	
		if(oflag[0]>1) fwrite(buf, 1, nrange*2, fp_out);

		/* skip rest of the range per record */ 
		fseek(fp_in, nremnant_byte_per_record, SEEK_CUR); 
	}
	fclose(fp_in);
	if(oflag[0]>1) fclose(fp_out);

	/* Nominal Mean and DC bias compensation*/
	printf("  Nominal Mean and DC bias compensation.\n");
	for(i=0; i<nazimuth; i++)
	{
		ix = i*nrange_fft*2; 
		for(j=0; j<nrange; j++)
		{
			data[ix+2*j] -= (nominal_mean_I + DC_bias_I);
			data[ix+2*j+1] -= (nominal_mean_Q + DC_bias_Q);
		}
	}

}


