/* memory allocation for vectors and matrices */
/* modified from nrutil.c of Numerical Recipes in C */
/* index notation follows C convention */

/*#include <stdlib.h>*/

unsigned char *ucvector(int n)
{
        unsigned char *v;

        v = (unsigned char *)calloc((size_t) n, (size_t) sizeof( unsigned char));
        if(!v)
        {       printf("malloc failure in unsigned char vector\n");
                exit(0);
        }

        return v;
}

char *cvector(int n)
{
        char *v;

        v = (char *)calloc((size_t) n, (size_t) sizeof(char));
        if(!v)
        {       printf("malloc failure in char vector\n");
                exit(0);
        }

        return v;
}

int *ivector(int n)
{
        int *v;

        v = (int *)calloc((size_t) n, (size_t) sizeof(int));
        if(!v)
        {       printf("malloc failure in int vector\n");
                exit(0);
        }
        return v;
}

float *fvector(int n)
{
	float *v;

	v = (float *)calloc((size_t) n, (size_t) sizeof(float));
	if(!v)
	{	printf("malloc failure in float vector\n");
		exit(0);
	}	
	return v;
}

double *dvector(int n)   
{
        double *v;               
 
        v = (double *)calloc((size_t) n, (size_t) sizeof(double));       
        if(!v)
        {       printf("malloc failure in double vector\n");     
                exit(0);       
        }
        return v;      
}

int **imatrix(int nrow, int ncol)
{
        int i;
        int **m;

        /* allocate pointers to rows */
        m=(int **)calloc((size_t) nrow, (size_t) sizeof(int*));
        if(!m)
        {       printf("malloc failure 1 in int matrix\n");
                exit(0);
        }

        /* allocate rows and set pointers to them */
        m[0]=(int *)calloc((size_t) nrow*ncol, (size_t)sizeof(int));
        if(!m[0])
        {       printf("malloc failure 2 in int matrix\n");
                exit(0);
        }

        for(i=1; i< nrow; i++) m[i] = m[i-1] + ncol;

        return m;
}

float **fmatrix(int nrow, int ncol)
{
        int i;
        float **m;

        /* allocate pointers to rows */
        m=(float **)calloc((size_t) nrow, (size_t) sizeof(float*));
        if(!m)
        {       printf("malloc failure 1 in float matrix\n");
                exit(0);
        }

        /* allocate rows and set pointers to them */
        m[0]=(float *)calloc((size_t) nrow*ncol, (size_t) sizeof(float));
        if(!m[0])
        {       printf("malloc failure 2 in float matrix\n");
                exit(0);
        }

        for(i=1; i< nrow; i++) m[i] = m[i-1] + ncol;

        return m;
}

double **dmatrix(int nrow, int ncol)
{
	int i;
	double **m;

	/* allocate pointers to rows */
	m=(double **)calloc((size_t) nrow, (size_t) sizeof(double*));
	if(!m)
        {       printf("malloc failure 1 in double matrix\n");     
                exit(0);       
        }
	
	/* allocate rows and set pointers to them */
	m[0]=(double *)calloc((size_t) nrow*ncol, (size_t) sizeof(double));
        if(!m[0])
        {       printf("malloc failure 2 in double matrix\n");     
                exit(0);       
        }
	
	for(i=1; i< nrow; i++) m[i] = m[i-1] + ncol;
	
	return m;
}
