/************************************************************
		range_chirp.c
	   	
  Prepare the conjugate of the transmitted range chirp signal. 

	1. FFT compatible format (periodic)
	2. Demodulated chirp signal (Middle of the chirp 
	   centred at origin, t=0)


	<< Conjugate of the demolulated Chirp >>
	   
	   s(t) = exp[-pi*K*t*t], |t|<tau_p/2
	  
	   where     K: chirp rate
	 	 tau_p: chirp duration	


  		   << Data Structure>>

	    *****...****0   ... 0 ...     0*****...****
	    -------------------------------------------
fast-	    |          |		   |          |  
 time(t):   0    chirp_duration/2  -chirp_duration/2 -delta_t		
 
range_chirp |          |                   |          |
_data[i]:   0      chirp_end	    chirp_start  nrange_fft-1 


by Dr. Hoonyol Lee
Department of Geophysics
Kangwon National University
Initiated on 7 May 2004
*************************************************************/

void range_chirp()
{
	int i, chirp_end, chirp_start; 
	double t, phase;

	printf(" range_chirp:\n");

	range_chirp_data = dvector(nrange_fft*2);

	chirp_end = (int)(chirp_duration/2./delta_t + 0.5);
	chirp_start = nrange_fft + 
		(int)(chirp_duration/(-2.)/delta_t + 0.5);

	/* first half of nrange_fft */ 
	for(i=0; i<chirp_end; i++) 
	{
		t = i*delta_t; 	
		phase = pi*chirp_rate*t*t;
		range_chirp_data[2*i]=cos(phase);
		range_chirp_data[2*i+1]= (-1.)*sin(phase); /* conjugate */
	}

	/* next half of nrange_fft */
	for(i=chirp_start; i<nrange_fft; i++) 
	{
		t = (i-nrange_fft)*delta_t; 	
		phase = pi*chirp_rate*t*t;
		range_chirp_data[2*i]=cos(phase);
		range_chirp_data[2*i+1]= (-1.)*sin(phase); /* conjugate */
	}

}

 	
