/**********************************************
	range_migration.c

	delta_R = R-Rc= 
		-lambda/2*f_Dc*(s-s_c)
		-lambda*f_r/4*(s-s_c)^2
	     
	       = -lambda/2*(s-s_c)(f_Dc+f_r/2*(s-s_c))
 
	putting	s-s_c = (f-f_Dc)/f_R gives:

	delta_R = -lambda/4*(f^2-f_Dc^2)/f_r 

by Dr. Hoonyol Lee
Dept. of Geophysics
Kangwon National University

azimuth ambituity : 1 Dec 2004
start: 22 May 2004
***********************************************/

void range_migration()
{
 	int i, j, ix, jx, jm;  /* jm = 'j migrated' */
	int idelta_pixel;
	double delta_R, delta_f, delta_pixel;
	double f; /*azimuth frequency*/
	double delta_R_max, delta_R_min;
	int idelta_pixel_max, idelta_pixel_min;
	double *buf;	
	double f_Dc, f_Dc0, f_Dc1, f_Dc_middle, f_r, prf, h_prf;
	double az_ambiguity, f_0, f_Dc_remnant; 
	int f_Dc_azpixel;

	printf(" range_migration:\n");

	f_Dc0=Doppler_centroid0;
	f_Dc1=Doppler_centroid1;
	
	f_Dc_middle = Doppler_centroid;
	f_r = Doppler_rate;
	prf = pulse_repetition_frequency;
	h_prf = prf/2.;
	az_ambiguity = floor(f_Dc_middle/prf+0.5); 
	f_0 = prf*az_ambiguity;
	f_Dc_remnant = fmod(f_Dc_middle, prf);                 

	printf("  Doppler_centroid = az_ambiguity*prf + f_Dc_remnant\n");
	printf("  %lf = %d*%lf + %lf, f_0=%lf\n", f_Dc_middle, (int)az_ambiguity,
		prf, f_Dc_remnant, f_0); 

	/* Regarding azimuth ambiguity, f_0 is the actual frequency 
	of the first line in the azimuth FFTed image after range compression
	(1 Dec 2004) */
	
	buf = dvector(nrange_fft*2);

	delta_f = pulse_repetition_frequency/nazimuth_fft;

	delta_R_max=-1.e8;
	idelta_pixel_max=-100000000;
	delta_R_min=1.e8;
	idelta_pixel_min=100000000;

	for(i=0; i<nazimuth_fft; i++)
	{
	   f= f_0 + i*delta_f;
	   ix=i*nrange_fft*2;
	   for(j=0; j<nrange_fft; j++)
	   {
		f_Dc = f_Dc0 + f_Dc1*j;

	   	if((f-f_Dc)> h_prf) f -= prf;
	   	if((f-f_Dc)< -1.*h_prf) f += prf;
	   	delta_R = -1.*wavelength/4.*(f*f-f_Dc*f_Dc)/f_r; 
	   	delta_pixel = delta_R/range_pixel_size;
	   	idelta_pixel = (int)(delta_pixel+0.5);

	   	if(delta_R>delta_R_max) delta_R_max = delta_R;
	   	if(delta_R<delta_R_min) delta_R_min = delta_R;
	   	if(idelta_pixel>idelta_pixel_max) idelta_pixel_max=idelta_pixel;
	   	if(idelta_pixel<idelta_pixel_min) idelta_pixel_min=idelta_pixel;

	   	/*printf("%d %lf %lf %lf\n", i, f, delta_R, delta_pixel);
	   	*/
	
		jm=j-idelta_pixel; /* negative: 1 Dec 2004 */
		
		/* cyclic shift: 1 Dec 2004 */
		if(jm<0)
		{
		   buf[2*j] = data[ix+2*(jm+nrange_fft)];
		   buf[2*j+1] = data[ix+2*(jm+nrange_fft)+1];
		}

		if(jm>=nrange_fft)
		{
		   buf[2*j] = data[ix+2*(jm-nrange_fft)];
		   buf[2*j+1] = data[ix+2*(jm-nrange_fft)+1];
		}

		if(jm>=0 && jm<nrange_fft)	
		{
		   buf[2*j] = data[ix+2*jm];
		   buf[2*j+1] = data[ix+2*jm+1];
	  	}
	   }
	   for(j=0; j<nrange_fft*2; j++) data[ix+j]=buf[j];

	}
	printf("   maximum migration= %lf m, %d pixels.\n", delta_R_max, idelta_pixel_max);
	printf("   minimum migration= %lf m, %d pixels.\n", delta_R_min, idelta_pixel_min);

	free(buf);

	/* Align Doppler Centroid at f=f_0 */
	buf=dvector(nazimuth_fft*2);
	for(i=0; i<nrange_fft; i++)
	{
	   f_Dc = f_Dc0 + f_Dc1*i;
	   f_Dc_azpixel= (int)((f_Dc - f_0)/prf*nazimuth_fft+0.5);
	   while(f_Dc_azpixel>=nazimuth_fft) f_Dc_azpixel -= nazimuth_fft;
	   while(f_Dc_azpixel<0) f_Dc_azpixel += nazimuth_fft;
	   
	   for(j=0; j<nazimuth_fft; j++)
	   {
		jx = j+f_Dc_azpixel;
		if(jx>=nazimuth_fft) jx -= nazimuth_fft;
		jx *= (nrange_fft*2);
		buf[2*j]=data[jx+2*i];
		buf[2*j+1]=data[jx+2*i+1];
	   }

	   for(j=0; j<nazimuth_fft; j++)
	   {
		jx=nrange_fft*2*j;
		data[jx+2*i]=buf[2*j];
		data[jx+2*i+1]=buf[2*j+1];
	   }
	   
	  /*printf("%d ", i);
	  */
	}
	free(buf);

}


