/**********************************************
	range_migration.c

	delta_R = R-Rc= 
		-lambda/2*f_Dc*(s-s_c)
		-lambda*f_r/4*(s-s_c)^2
	     
	       = -lambda/2*(s-s_c)(f_Dc+f_r/2*(s-s_c))
 
	putting	s-s_c = (f-f_Dc)/f_R gives:

	delta_R = -lambda/4*(f^2-f_Dc^2)/f_r 

by Dr. Hoonyol Lee
Dept. of Geophysics
Kangwon National University

azimuth ambituity : 1 Dec 2004
start: 22 May 2004
***********************************************/

void range_migration()
{
 	int i, j, ix, jm;  /* jm = 'j migrated' */
	int idelta_pixel;
	double delta_R, delta_f, delta_pixel;
	double f; /*azimuth frequency*/
	double delta_R_max, delta_R_min;
	int idelta_pixel_max, idelta_pixel_min;
	double *buf;	
	double f_Dc, f_r, prf;
	double az_ambiguity, f_0, f_Dc_remnant; 

	printf(" range_migration:\n");
	
	f_Dc = Doppler_centroid;
	f_r = Doppler_rate;
	prf = pulse_repetition_frequency;
	az_ambiguity = floor(f_Dc/prf); 
	f_0 = prf*az_ambiguity;
	f_Dc_remnant = fmod(f_Dc, prf);                 

	printf("  Doppler_centroid = az_ambiguity*prf + f_Dc_remnant\n");
	printf("  %lf = %d*%lf + %lf, f_0=%lf\n", f_Dc, (int)az_ambiguity,
		prf, f_Dc_remnant, f_0); 

	/* Regarding azimuth ambiguity, f_0 is the actual frequency 
	of the first line in the azimuth FFTed image after range compression
	(1 Dec 2004) */
	
	buf = dvector(nrange_fft*2);

	delta_f = pulse_repetition_frequency/nazimuth_fft;

	/* half_prf frequency less than f_dc */	
	delta_R_max=-1.e8;
	idelta_pixel_max=-100000000;
	delta_R_min=1.e8;
	idelta_pixel_min=100000000;
	for(i=0; i<nazimuth_fft/2; i++)
	{
	   f= f_0 + i*delta_f;
	   ix=i*nrange_fft*2;
	   delta_R = -1.*wavelength/4.*(f*f-f_Dc*f_Dc)/f_r; 
	   delta_pixel = delta_R/range_pixel_size;
	   idelta_pixel = (int)(delta_pixel+0.5);

	   if(delta_R>delta_R_max) delta_R_max = delta_R;
	   if(delta_R<delta_R_min) delta_R_min = delta_R;
	   if(idelta_pixel>idelta_pixel_max) idelta_pixel_max = idelta_pixel;
	   if(idelta_pixel<idelta_pixel_min) idelta_pixel_min = idelta_pixel;

	   /*printf("%d %lf %lf %lf\n", i, f, delta_R, delta_pixel);
	   */	
	   for(j=0; j<nrange_fft; j++)
	   {
		jm=j-idelta_pixel; /* negative: 1 Dec 2004 */
		if(jm>=0 && jm<nrange_fft)	
		{
		   buf[2*j] = data[ix+2*jm];
		   buf[2*j+1] = data[ix+2*jm+1];
	  	} 
	   }
	   for(j=0; j<nrange_fft*2; j++) data[ix+j]=buf[j];
		
	}

	/* half_prf frequency larger than f_Dc */	
	for(i=nazimuth_fft/2; i<nazimuth_fft; i++)
	{
	   f= f_0 + (i-nazimuth_fft)*delta_f;
	   ix=i*nrange_fft*2;
	   delta_R = -1.*wavelength/4.*(f*f-f_Dc*f_Dc)/f_r; 
	   delta_pixel = delta_R/range_pixel_size;
	   idelta_pixel = (int)(delta_pixel+0.5);

	   if(delta_R > delta_R_max) delta_R_max = delta_R;
	   if(delta_R < delta_R_min) delta_R_min = delta_R;
	   if(idelta_pixel > idelta_pixel_max) idelta_pixel_max = idelta_pixel;
	   if(idelta_pixel < idelta_pixel_min) idelta_pixel_min = idelta_pixel;

	   /* printf("%d %lf %lf %lf\n", i, f, delta_R, delta_pixel);
	   */ 
	   for(j=0; j<nrange_fft; j++)
	   {
		jm=j-idelta_pixel; /* minus: 1 Dec 2004 */
		if(jm>=0 && jm<nrange_fft)	
		{
		   buf[2*j] = data[ix+2*jm];
		   buf[2*j+1] = data[ix+2*jm+1];
	  	} 
	   }
	   for(j=0; j<nrange_fft*2; j++) data[ix+j]=buf[j];

	}
	printf("   maximum migration= %lf m, %d pixels.\n", delta_R_max, idelta_pixel_max);
	printf("   minimum migration= %lf m, %d pixels.\n", delta_R_min, idelta_pixel_min);

}


